class Selection():
    def __init__(self, soup, selectors):
        self.data = [soup]
        self.selectors = selectors

    def get(self):
        """
            Get the first specified part/tag of a web page.
        """

        for selector in self.selectors:
            if selector["attr"]:
                if selector["text"]:
                    self.data = self.data[0].find(selector["tag"], attrs = {selector["attr"], selector["value"]}).text
                elif selector["attrt"]:
                    self.data = self.data[0].find(selector["tag"], attrs = {selector["attr"], selector["value"]})[selector["attrt"]]
                else:
                    self.data = self.data[0].find(selector["tag"], attrs = {selector["attr"], selector["value"]})
            else:
                if selector["text"]:
                    self.data = self.data[0].find(selector["tag"]).text
                elif selector["attrt"]:
                    self.data = self.data[0].find(selector["tag"])[selector["attrt"]]
                else:
                    self.data = self.data[0].find(selector["tag"])

        return self.data

    def getall(self):
        """
            Get all the specified parts/tags of a web page.
        """

        for selector in self.selectors:
            if selector != self.selectors[-1]:
                if selector["attr"]:
                    if selector["text"]:
                        raise Exception("Text is not allowed to be taken in any of the tag selectors except for the last one.")
                    elif selector["attrt"]:
                        raise Exception("Attribute is not allowed to be taken in any of the tag selectors except for the last one.")
                    else:
                        self.data = self.data[0].find_all(selector["tag"], attrs = {selector["attr"], selector["value"]})
                else:
                    if selector["text"]:
                        raise Exception("Text is not allowed to be taken in any of the tag selectors except for the last one.")
                    elif selector["attrt"]:
                        raise Exception("Attribute is not allowed to be taken in any of the tag selectors except for the last one.")
                    else:
                        self.data = self.data[0].find_all(selector["tag"])
            else:
                if selector["attr"]:
                    if selector["text"]:
                        self.data[0] = [tag.text for tag in self.data[0].find_all(selector["tag"])]
                    elif selector["attrt"]:
                        self.data[0] = [tag[selector["attrt"]] for tag in self.data[0].find_all(selector["tag"])]
                    else:
                        self.data[0] = [tag for tag in self.data[0].find_all(selector["tag"])]
                else:
                    if selector["text"]:
                        self.data[0] = [tag.text for tag in self.data[0].find_all(selector["tag"])]
                    elif selector["attrt"]:
                        self.data[0] = [tag[selector["attrt"]] for tag in self.data[0].find_all(selector["tag"])]
                    else:
                        self.data[0] = [tag for tag in self.data[0].find_all(selector["tag"])]

        return self.data[0]


import requests
from bs4 import BeautifulSoup, SoupStrainer
from .soup import Soup

class Scraper():
    """
        Used for making a scraper.
    """

    def __init__(self):
        self.session = requests.Session()

    def strainer(self, selector: str = None, tag: str = None, attrs = None):
        """
            Used for creating a soup strainer.

            Required Parameters
            -------------------
            `selector`: CSS selector for creating a strainer.
            `tag`: Tag for creating a strainer.
            NOTE: `selector` and `tag` both should not be given. Give either
            `selector` or `tag`.

            Optional Parameters
            -------------------
            `attrs`: Attribute of a tag with its value used as a criteria (Use
            only when `tag` parameter is used).
        """

        if selector:
            if tag or attrs:
                raise Exception("CSS selector and tag/attribute given. Please provide either CSS selector or tag (and attribute it's optional).")
            else:
                selector = selector.replace("[", " ")
                selector = selector.replace("]", "")
                split = selector.split(" ")
                if len(split) == 2:
                    attr_value = split[1].split("*=")
                    strainer = SoupStrainer(split[0], attrs = {attr_value[0], attr_value[1]})
                else:
                    strainer = SoupStrainer(split[0])
        else:
            strainer = SoupStrainer(tag, attrs = attrs)

        return strainer

    def scrape(self, url, strainer = None):
        """
            Used for scraping data from a web page.

            Required Parameters
            -------------------
            `url`: URL of the web page.

            Optional Parameters
            -------------------
            `strainer`: Soup strainer for parsing only the required part of the
            web page.
        """

        data = self.session.get(url)
        content = BeautifulSoup(data.content, "lxml", parse_only = strainer)
        soup = Soup(content)
        return soup


from .selection import Selection

class Soup():
    def __init__(self, soup):
        self.soup = soup
    
    def css(self, selector):
        """
            (Scrapy-style)
            Get CSS selector to get the specified part/tag of the web page.

            Required Parameters
            -------------------
            `selector`: CSS selector to get the specified part/tag of the web
            page.
        """

        selectors = []

        split = selector.split(" ")
        for i in split:
            i = i.replace("[", " ")
            i = i.replace("]", "")
            i_split = i.split(" ")
            if len(i_split) == 2:
                further = i_split[1].split("::")
                attr_value = further[0].split("*=")
                if len(further) == 2:
                    if further[1] == "text":
                        selectors.append({
                            "tag": i_split[0],
                            "attr": attr_value[0],
                            "value": attr_value[1],
                            "text": True,
                            "attrt": None
                        })
                    else:
                        attr = further[1].replace("attr(", "")
                        attr = attr.replace(")", "")
                        selectors.append({
                            "tag": i_split[0],
                            "attr": attr_value[0],
                            "value": attr_value[1],
                            "text": False,
                            "attrt": attr
                        })
                else:
                    selectors.append({
                        "tag": i_split[0],
                        "attr": attr_value[0],
                        "value": attr_value[1],
                        "text": False,
                        "attrt": None
                    })
            else:
                further = i.split("::")
                if len(further) == 2:
                    if further[1] == "text":
                        selectors.append({
                            "tag": further[0],
                            "attr": None,
                            "value": None,
                            "text": True,
                            "attrt": None
                        })
                    else:
                        attr = further[1].replace("attr(", "")
                        attr = attr.replace(")", "")
                        selectors.append({
                            "tag": further[0],
                            "attr": None,
                            "value": None,
                            "text": False,
                            "attrt": attr
                        })
                else:
                    selectors.append({
                        "tag": further[0],
                        "attr": None,
                        "value": None,
                        "text": False,
                        "attrt": None
                    })

        selection = Selection(self.soup, selectors)
        return selection

    def find(self, tag: str, attrs = None):
        """
            (BeautifulSoup-style)
            Getting the first specified part/tag of a web page.

            Required Parameters
            -------------------
            `tag`: Tag for getting the first specified part/tag of a web page.

            Optional Parameters
            -------------------
            `attrs`: Attribute and value used as a criteria.
        """

        data = self.soup.find(tag, attrs = attrs)
        data_soup = Soup(data)

        return data

    def find_all(self, tag: str, attrs = None):
        """
            (BeautifulSoup-style)
            Getting all the specified parts/tags of a web page.

            Required Parameters
            -------------------
            `tag`: Tag for getting all the specified parts/tags of a web page.

            Optional Parameters
            -------------------
            `attrs`: Attribute and value used as a criteria.
        """

        data = self.soup.find_all(tag, attrs = attrs)
        data_soup = Soup(data)

        return data


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="scrapist",
    version="1.0.0",
    author="Areen-Rath",
    description="A fast web scraper for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    py_modules=["scraping"],
    package_dir={'':'.'},
    install_requires=["requests", "beautifulsoup4", "lxml"]
)